% NGLIOBJ    Ng and Li's Standard Objective Function.
%
% F = NGLIOBJ(X,NL) calculates the objective function value of the
% Ng and Li's test problem at X.  The parameters of the test problem 
% are stored in the structure array NL with fields:
%   n: integer,  and
%   a, p, q, s, d, v, wLB, yLB, k: column n-vectors.
% For standard test problems 1--300, NL are stored in the m-files
% NLnnn.M, where nnn is the problem ID.
% For example, we can find the objective function value of the problem 
% number 1 at the point X = [0;0] as follows: 
%    fval = ngliobj([0;0], ngli000).
%
% [F,DF] = NGLIOBJ(X,NL) calculates the objective function value and
% the gradient column vector of the Ng and Li's test problem at X 
% with parameters in NL.
%
% [F,DF,DDF] = NGLIOBJ(X,NL) calculates the objective function value,
% the gradient column vector and the Hessian matrix of the Ng and Li's
% test problem at X with parameters in NL.
% For example, we can find a local minimizer and its value for the 
% problem number 1 by FMINUNC with initial point [0;0] as follows:
%    fun      = @(x)ngliobj(x, ngli001);
%    x0       = [0;0];
%    options  = optimset('GradObj','on', 'Hessian','on');
%    [x,fval] = fminunc(fun, x0, options);



function [h, dh, ddh] = ngliobj(w, NL)


if nargin ~= 2
  error('[h, dh, ddh] = ngliobj(w, NL)');
end


if isempty(NL.wLB)
  % Same as  K = eye(n); wLB = zeros(n,1); yLB = zeors(n,1);

  D = diag(NL.d);
  H = eye(NL.n) - 2 * (NL.v * NL.v');

  x = D * H * w;
  h = NL.a' * (x.^4 + 4*NL.p.*x.^3 + 6*NL.q.*x.^2 + NL.s.*x);

  if nargout >= 2
    df = NL.a .* (4*x.^3 + 12*NL.p.*x.^2 + 12*NL.q.*x + NL.s);
    dh = H * D * df;
  end

  if nargout >= 3
    ddf = diag( 12*NL.a.*(x.^2 + 2*NL.p.*x + NL.q) );
    ddh = H * D * ddf * D * H;
  end

else

  D = diag(NL.d);
  K = diag(NL.k);
  H = eye(NL.n) - 2 * (NL.v * NL.v');

  y = K * (w - NL.wLB) + NL.yLB;
  x = D * H * y;
  h = NL.a' * (x.^4 + 4*NL.p.*x.^3 + 6*NL.q.*x.^2 + NL.s.*x);

  if nargout >= 2
    df = NL.a .* (4*x.^3 + 12*NL.p.*x.^2 + 12*NL.q.*x + NL.s);
    dg = H * D * df;
    dh = K * dg;
  end

  if nargout >= 3
    ddf = diag( 12*NL.a.*(x.^2 + 2*NL.p.*x + NL.q) );
    ddg = H * D * ddf * D * H;
    ddh = K * ddg * K;
  end
end
